function [S,ST]=Fast(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction)
%==============================================================
%                     fonction Fast
% INPUTS : 
%    NbCal : nb de repetition du calcul
%    locPrem : vaut 1 si calcul des indices de premier ordre, 0 sinon
%    locTot : vaut 1 si calcul des indices totaux, 0 sinon
%    locTout : vaut 1 si calcul des indices  tout ordre, 0 sinon
%    p : nb de variables d'entree
%    n : nb de simulation de Monte-Carlo
%    VVarEntree : tableau contenant les info sur les variables d'entree
%                  (numero de la var, loi, parametre 1, param 2)
%    FFonction : string contenant la focntion du modele
%
% OUTPUTS :
%    S : tableau contenant les indices de premier ordre
%             (numero indice, moyenne sur NbCal, ecart-type)
%    ST : tableau contenant les indices totaux
%             (numero indice, moyenne sur NbCal, ecart-type)
%
% Julien JACQUES, avril 2006
%==============================================================
    S=[];ST=[];
    c1=1./pi; 
    M=4;
    if locPrem==1
        % INITIALISATION
        % frequences de Saltelli&Co 1999
        if p<6, Frequences=[11;21;27;35;39];  end
        if p==6,  Frequences=[1;21;31;37;45;49];  end
        if p==7,  Frequences=[17;39;59;69;75;83;87];  end
        if p==8,  Frequences=[23;55;77;97;107;113;121;125];  end
        if p>8
            uiwait(warndlg('FAST not available for more than 8 inputs'));
            return
        end
        % Frequences associees aux parametres X1,X2,...,Xp   
        w=(Frequences(1:1:p))';
        % VARIABLES
        y=zeros(n,1);
        x=zeros(n,p);
        D=zeros(1,p);
        S=zeros(1,p);
        Spectre=zeros(n,p);
        % PARAMETRES D'ENTREE
        k=(1:1:n)';     
        s=(2*pi)*(k-1)/n;
        for i=1:p
            x(:,i)=0.5+c1*asin(sin(w(i)*s));
        end
        % TRANSFO DES PARAMETRES D'ENTREE
        for i=1:p
            loi=VVarEntree(i,2);
            p1=VVarEntree(i,3);
            p2=VVarEntree(i,4);
            x(:,i)=unif2loi(x(:,i),loi,p1,p2);
        end
        % CALCUL DE Y=F(X)
        y=[];
        y=eval(FFonction);
        % TRANSFORMEE DE FOURIER
        F=fft2(y(M:n));
        T=max(size(F));
        % COEFFICIENTS DE FOURIER
        a0=F(1)/T;
        for i=1:T-1
          a(i)=real(F(i+1))/T;
          b(i)=imag(F(i+1))/T;
        end
        % SPECTRE DE F(X)
        Spectre=a(1:(T-1)/2).^2+b(1:(T-1)/2).^2;
        % VARIANCE TOTALE
        Dtotal=2*sum(Spectre);
        for i=1:p
        % TRANSFORMEE DE FOURIER
       for j=1:w(i):T
            FF(j,i)=F(j);
       end
       t(i)=max(size(FF(:,i)));
        % COEFFICIENTS DE FOURIER ASSOCIES AUX X1...Xp
       aa0(i)=FF(1,i)/t(i);
       for j=1:1:t(i)-1
         aa(j,i)=real(FF(j+1,i))/t(i);
         bb(j,i)=imag(FF(j+1,i))/t(i);
        end
        % SPECTRES PARTIELS
        SpectrePartiel(1:(t(i)-1)/2,i)=aa(1:(t(i)-1)/2,i).^2+bb(1:(t(i)-1)/2,i).^2; 
        % VARIANCES PARTIELLES
        D(i)=2*sum(SpectrePartiel(:,i));
        end
        % INDICES FAST
        S=D/Dtotal;
        result=[(1:p)' S'];
        S=result
    end
 if locTot==1
 %-------------------------------------------------------  
 % INDICES Extended FAST  (totaux)
 %-------------------------------------------------------  
    w=[80,1,5,7,9,11,13,17,19,23,29]; 
    ST=[];
    k=(1:1:n)';     
    s=(2*pi)*(k-1)/n;
    SpectreT=[];
    for i=1:p
      y=zeros(n,1);
      x=zeros(n,p);
      D=0;T=0;
      for j=1:p
        x(:,j)=0.5+c1*asin(sin(w(j+1)*s));
      end
      x(:,i)=0.5+c1*asin(sin(w(1)*s));
      % TRANSFO DES PARAMETRES D'ENTREE
      for i=1:p
        loi=VVarEntree(i,2);
        p1=VVarEntree(i,3);
        p2=VVarEntree(i,4);
        x(:,i)=unif2loi(x(:,i),loi,p1,p2);
      end
      % CALCUL DE Y=F(X)
	  y=eval(FFonction);
      % TRANSFORMEE DE FOURIER
      F=fft2(y(M:n));
      T=max(size(F));
      % COEFFICIENTS DE FOURIER
	  for j=1:T-1
        a(j)=real(F(j+1))/T;
        b(j)=imag(F(j+1))/T;
      end
      % SPECTRE 
      Spectre=(a(1:(T-1)/2).^2)+(b(1:(T-1)/2).^2);
      % VARIANCE TOTALE
      Dtotal=2*sum(Spectre);
      D=2*sum(Spectre(1:w(1)/2));
      % INDICES FAST GLOBAL
      ST=[ST,1-D/Dtotal];
    end
  result=[(1:p)' ST'];
  ST=result
 end