function [S]=Mckay(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction)
%==============================================================
%                     fonction McKay
% INPUTS : 
%    NbCal : nb de repetition du calcul
%    locPrem : vaut 1 si calcul des indices de premier ordre, 0 sinon
%    locTot : vaut 1 si calcul des indices totaux, 0 sinon
%    locTout : vaut 1 si calcul des indices  tout ordre, 0 sinon
%    p : nb de variables d'entree
%    n : nb de simulation de Monte-Carlo
%    VVarEntree : tableau contenant les info sur les variables d'entree
%                  (numero de la var, loi, parametre 1, param 2)
%    FFonction : string contenant la focntion du modele
%
% OUTPUTS :
%    S : tableau contenant les indices de premier ordre
%             (numero indice, moyenne sur NbCal, ecart-type)
%
% Julien JACQUES, avril 2006
%==============================================================
S=[];        %matrice des indices
D=[];        %matrice des variance partielle
if n>100
    disp('too high iterations number for McKay computation. We fix it to 100');
    n=100;
end
r=floor(n/2);   %nb de replications pour le r-LHS sampling
for kkk=1:NbCal
% simulation des entrees
    X=zeros(n*r,p);
    % simulation des n premieres lignes
    for i=1:p
        loi=VVarEntree(i,2);
        p1=VVarEntree(i,3);
        p2=VVarEntree(i,4);
        if loi==1 %simulation uniforme entre p1 et p2
            for j=1:n
                X(j,i)=p1+(p2-p1)/n*(j-1+0.5);
                %X(j,i)=rand(1,1)*(p2-p1)-p1;
            end
        end
        if not(loi==1) 
            uiwait(warndlg('McKay method not available for non uniform inputs'));
            return;    
        end
    end
    % replication des n premieres lignes
    for k=1:r-1
        for i=1:p       
            permut=randperm(n);
            loc=X(1:n,i);
            X(n*k+1:n*k+n,i)=loc(permut);
        end
    end
    % CALCUL DE Y=F(X)
    y=[];
    x=X;
    y=eval(FFonction);
    % CALCUL des moyennes sur les replications
    for i=1:p
       x=X; 
       for l=1:n
            for m=1:r
                x(l+n*(m-1),i)=x(l,i);
            end
        end
        yjp=zeros(1,n);
        yp=[];
        for j=1:n
            for k=1:r
                for l=1:n
                    for m=1:r
                        x(l+n*(m-1),i)=x(l,i);
                    end
                end
                loc=eval(FFonction);
                yp(k)=loc(j+(k-1)*n);
            end 
            yjp(j)=sum(yp)/r;
        end
        vloc=0;
        for j=1:n
            vloc1=0;
            for k=j:n:r*n
                vloc1=vloc1+(y(k)-mean(y)).^2;
            end
            vloc=vloc+vloc1;
        end
        varY=vloc/(n*r);
        % CALCUL des indices
        loc=0;
        for j=1:n
            loc=loc+(yjp(j)-mean(y)).^2;
        end
        s1=loc/n;
        loc=0;
        for j=1:n
            loc1=0;
            ypp=[];
            for k=1:r
                loc2=0;
                for l=1:n
                    for m=1:r
                        x(l+n*(m-1),i)=x(l,i);
                    end
                end
                loc2=eval(FFonction);
                ypp(k)=(loc2(j+(k-1)*n)-yjp(j)).^2;
                loc1=loc1+ypp(k);
            end 
            loc=loc+loc1;
        end
        s2=loc/(n*r^2);
        S(kkk,i)=(s1-s2)/varY;
    end
end
result=[(1:p)' mean(S)' sqrt(var(S))'];
S=result