function [retour,retourT,retourTout]=Sobol(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction,locSampling)
%==============================================================
%                     fonction Sobol
% INPUTS : 
%    NbCal : nb de repetition du calcul
%    locPrem : vaut 1 si calcul des indices de premier ordre, 0 sinon
%    locTot : vaut 1 si calcul des indices totaux, 0 sinon
%    locTout : vaut 1 si calcul des indices  tout ordre, 0 sinon
%    p : nb de variables d'entree
%    n : nb de simulation de Monte-Carlo
%    VVarEntree : tableau contenant les info sur les variables d'entree
%                  (numero de la var, loi, parametre 1, param 2)
%    FFonction : string contenant la focntion du modele
%    locSampling : variable prenant les valeur 1 si le schema 
%                  d'echantillonage est CrudeMC, 2 pour QMC
%
% OUTPUTS :
%    retour : tableau contenant les indices de premier ordre
%             (numero indice, moyenne sur NbCal, ecart-type)
%    retourT : tableau contenant les indices totaux
%             (numero indice, moyenne sur NbCal, ecart-type)
%    retourTout : tableau contenant les indices  tout ordre
%             (numero indice, moyenne sur NbCal, ecart-type)
%
% Julien JACQUES, avril 2006
%==============================================================
S=[];         %matrice des indices
%lancement des Ncalcul calculs d'indice
for k=1:NbCal
    %simulation de deux matrices d'entrees u et v
    u=[];v=[];
    %tri en fonction de la colonne var de la matrice d'entrep1
     if locSampling==2
         SobolQMC(-1);
     end%intialisation pour QMC
    for i=1:p
        loi=VVarEntree(i,2);
        p1=VVarEntree(i,3);
        p2=VVarEntree(i,4);
        if locSampling==1
            u(:,i)=simule(loi,p1,p2,n,1,locSampling);
            v(:,i)=simule(loi,p1,p2,n,1,locSampling);
        else
          %----sobol Lptau Saltelli------
          uv=simule(loi,p1,p2,n,2,locSampling);
          permut=randperm(n);
          utmp=uv(:,1);
          u(:,i)=utmp(permut);%on permutte pour contrer le fait
          permut=randperm(n); %que les squences de Sobol simule
          utmp=uv(:,2);       %une fois sur deux un u<0.5 et la 
          v(:,i)=utmp(permut);%fois suivante un u>0.5
        end
    end
    %%initialisation de la matrice dont chaque ligne definit les indices
	%%relatifs aux variables d'entree des indices de Sobol
	%%par ex: ligne 0101 modelise l'indice S_24
	M=[];
	M=creer_matrice(p);
    %%initialisation du vecteur des variances partielles
	Di=[];DT=[];
    %%calcul de la reponse y=f(u)
	x=u;
    y=eval(FFonction);
    %calcul des moments d'y
	f0=mean(y);
    D=mean(y.^2)-f0^2;
    numind=[0];
    %calcul des indices par simulation de MonteCarlo
	for i=2:size(M,1)
	  liste=[];
	%ex: pour la ligne 0101 de M, j=[2 4]
	  j=find(M(i,:));
      ll=EcritIndice(j);
      numind=[numind;ll];
	%calcul des indices d'ordre 1
	  if length(j)==1 
		if locPrem==1
          somme=0;
		  x_star=v;
		  x_star(:,j)=u(:,j);
          x=x_star;
          somme=sum(y.*eval(FFonction));
		  Di(i)=somme/n-f0^2;
        end
      else
	%calcul des indices d'ordre superieur
		if (locTout==1)&(p<10)
            somme=0;
		    x_star=v;
		    for l = j
			    x_star(:,l)=u(:,l);
		    end
            x=x_star;
            somme=sum(y.*eval(FFonction));
		    Di(i)=somme/n-f0^2;
            liste=inverse_creer_matrice(j,p,M);
		    for l = liste
			    Di(i)=Di(i)-Di(l);
            end
        else
            Di(i)=0;%indice non calcules
        end
	  end
	  if locTout==1|locPrem==1 
          S(i,k)=Di(i)/D;
      end
  end
  %calcul des indices totaux
    if locTot==1
        numindT=[];
        for i=1:p
            x_star=u;
            somme=0;
            x_star(:,i)=v(:,i);
            x=x_star;
            somme=sum(y.*eval(FFonction));
            DT(i)=somme/n-f0^2;
            numindT=[numindT;i];
            ST(i,k)=1-DT(i)/D;
        end
    end
end
retourT=[];retourTout=[];retour=[];
if locPrem==1&locTout==0
    S=S';
    res=[numind  mean(S)'  sqrt(var(S))'];
    res2=sortrows(res,1);
    res=res2(2:p+1,:);
    retour=res;
end
if locPrem==1&locTout==1
    S=S';
    res=[numind  mean(S)'  sqrt(var(S))'];
    res2=sortrows(res,1);
    res=res2(2:end,:);
    retour=res;
end
if locTot==1
    ST=ST';
    resT=[numindT  mean(ST)'  sqrt(var(ST))'];
    retourT=sortrows(resT,1);
end
if locTout==1
    retourTout=[retour;retourT];
end

%--------------------------------------------------------------------
%               sous fonctions de Sobol
%--------------------------------------------------------------------
function [M]=creer_matrice(p)
%creation de la matrice dont chaque ligne definit les indices
%relatifs aux variables d'entree des indices de Sobol
%par ex: ligne 0101 modelise l'indice S_24
	if p<10
        N=2^p;
	    M=zeros(N,p);
	    for k=1:p
	        v=[ zeros(2^(k-1),1) ; ones(2^(k-1),1) ];
	        M(:,p-k+1)=repmat(v,2^(p-k),1);
	    end
    else
        %matrice identite
        M=[zeros(1,p) ; eye(p)];
    end
%--------------------------------------------------------------------
function [x]=fpart(xx)
%fonction qui replique la matrice de var-cov
	x=xx-floor(xx);
%--------------------------------------------------------------------
function [k]=EcritIndice(j)
%fonction qui fait correspondre 24 a [2 4]
    k=0;	
    for i=1:length(j)
       k=k+10^(length(j)-i)*j(i); 
    end    
%--------------------------------------------------------------------
function [liste]=inverse_creer_matrice(indice,p,M)
%fonction inverse de creer matrice, 
%par ex: a partir de la ligne 0101 elle donne[2 4]
    liste=[];
    bin=zeros(1,p);
	for i = indice
		bin(i)=1;
	end
	for i=2:2^p
		d=bin-M(i,:);
		mauvais=0;
		for j=1:p
			if d(j)<0, mauvais=1; end
		end
		if mauvais==0 
			if sum(d)~=0, liste=[liste i]; end
		end
	end