function varargout = SAinterface(varargin)
%    SAINTERFACE Application M-file for SAinterface.fig
%=============================================================
%  SAinterface est une interface graphique permettant
%  d'effectuer l'analyse de sensibilite d'un modele saisi
%  par l'utilisateur au clavier ou sous forme d'un fichier 
%  du type myfunction.m
%
%             Julien JACQUES avril 2006
%=============================================================

if nargin == 0  % LAUNCH GUI
	fig = openfig(mfilename,'reuse');
	% Use system color scheme for figure:
	set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
    %ajout des chemins vers sous fonctions
    if isunix
        addpath ./SAToolBox;
        addpath ./SAToolBox/GAMToolBox;
    else
        addpath SAToolBox;
        addpath SAToolBox\GAMToolBox;
    end
	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
    % definition des variables globales
    handles.NbVar=2;
    handles.NbPoint=10000;
    handles.NbCalcul=3;    
	handles.Fonction='';    
	handles.VarEntree=[1 1 0 1;2 1 0 1];
    handles.ResultatSobol=[];
	handles.ResultatSobolT=[];
	handles.ResultatSobolTout=[];
	handles.ResultatFast=[];
	handles.ResultatFastT=[];
	handles.ResultatSrc=[];
	handles.ResultatSrrc=[];
    handles.ResultatPcc=[];
	handles.ResultatPrcc=[];
	handles.ResultatGam=[];
	handles.ResultatMcKay=[];
    handles.ExistenceCalcul=0;
    handles.Correlation=0;
    handles.p_ind=handles.NbVar;
    guidata(fig, handles);
	if nargout > 0
		varargout{1} = fig;
	end
elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
	try
		[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
	catch
		disp(lasterr);
	end
end

%| ABOUT CALLBACKS:
%| GUIDE automatically appends subfunction prototypes to this file, and 
%| sets objects' callback properties to call them through the FEVAL 
%| switchyard above. This comment describes that mechanism.
%|
%| Each callback subfunction declaration has the following form:
%| <SUBFUNCTION_NAME>(H, EVENTDATA, HANDLES, VARARGIN)
%|
%| The subfunction name is composed using the object's Tag and the 
%| callback type separated by '_', ep1.g. 'slider2_Callback',
%| 'figure1_CloseRequestFcn', 'axis1_ButtondownFcn'.
%|
%| H is the callback object's handle (obtained using GCBO).
%|
%| EVENTDATA is empty, but reserved for future use.
%|
%| HANDLES is a structure containing handles of components in GUI using
%| tags as fieldnames, ep1.g. handles.figure1, handles.slider2. This
%| structure is created at GUI startup using GUIHANDLES and stored in
%| the figure's application data using GUIDATA. A copy of the structure
%| is passed to each callback.  You can store additional information in
%| this structure at GUI startup, and you can change the structure
%| during callbacks.  Call guidata(h, handles) after changing your
%| copy to replace the stored original so that subsequent callbacks see
%| the updates. Type "help guihandles" and "help guidata" for more
%| information.
%|
%| VARARGIN contains any extra arguments you have passed to the
%| callback. Specify the extra arguments by editing the callback
%| property in the inspector. By default, GUIDE sets the property to:
%| <MFILENAME>('<SUBFUNCTION_NAME>', gcbo, [], guidata(gcbo))
%| Add any extra arguments after the last argument, before the final
%| closing parenthesis.


%////////////////////////////////////////////////////////////////
%                           MENUS
%////////////////////////////////////////////////////////////////

% --------------------------------------------------------------------
function varargout = SousMenuReca_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.PPNomVar.
entree=handles.VarEntree;
[nvar,toto]=size(entree);
for i=1:nvar
    if entree(i,2)==1, entree(i,2)='U'; end
    if entree(i,2)==2, entree(i,2)='N'; end
    if entree(i,2)==3, entree(i,2)='E'; end
    if entree(i,2)==4, entree(i,2)='W'; end
    if entree(i,2)==5, entree(i,2)='G'; end
    if entree(i,2)==6, entree(i,2)='S'; end
    if entree(i,2)==7, entree(i,2)='B'; end
    if entree(i,2)==8, entree(i,2)='C'; end
    if entree(i,2)==9, entree(i,2)='L'; end
 end
loc=[repmat('X',nvar,1) num2str(entree(:,1)) repmat(':',nvar,1) entree(:,2) repmat('[',nvar,1) num2str(entree(:,3)) repmat(',',nvar,1) num2str(entree(:,4)) repmat(']',nvar,1)];
disp(loc);

% --------------------------------------------------------------------
function varargout = SousMenuGraph_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.PPNomVar.
loc=get(handles.SousMenuGraph,'Label');
if strcmp(loc,'Draw graphs')==1
    if handles.ExistenceCalcul==0
        errordlg('no sensitivity indices computed');
    else
        set(handles.TVisualGraph,'Visible','on');
        set(handles.FVisualGraph,'Visible','on');
        set(handles.TVisualGraphMethode,'Visible','on');
        set(handles.PPVisualGraphMethode,'Visible','on');
        set(handles.PBGraph,'Visible','on');
        set(handles.SousMenuGraph,'Label','no graphs');
    end
else
    axes(handles.GraphTot);
    cla;
    set(handles.GraphTot,'Visible','off');
    axes(handles.GraphPrem);
    cla;
    set(handles.GraphPrem,'Visible','off');
    set(handles.TVisualGraph,'Visible','off');
    set(handles.FVisualGraph,'Visible','off');
    set(handles.TVisualGraphMethode,'Visible','off');
    set(handles.PPVisualGraphMethode,'Visible','off');
    set(handles.PBGraph,'Visible','off');
    set(handles.SousMenuGraph,'Label','Draw graphs');
end

% --------------------------------------------------------------------
function varargout = SousMenuGraphSave_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.PPNomVar.
    set(handles.FSaveGraph,'Visible','on');
    set(handles.TSaveGraph,'Visible','on');
    set(handles.RBSaveGraphPrem,'Visible','on');
    set(handles.RBSaveGraphTot,'Visible','on');
    set(handles.TSaveGraph2,'Visible','on');
    set(handles.ESaveGraph,'Visible','on');
    set(handles.PBSaveGraph,'Visible','on');
    set(handles.RBSaveGraphEps,'Visible','on');
    set(handles.RBSaveGraphJpg,'Visible','on');
    set(handles.RBSaveGraphFig,'Visible','on');
    set(handles.RBSaveGraphPrem,'value',0);
    set(handles.RBSaveGraphTot,'value',0);
    nomfich=get(handles.EFichierResultat,'string');
    set(handles.ESaveGraph,'string',nomfich);
    set(handles.RBSaveGraphEps,'value',0);
    set(handles.RBSaveGraphJpg,'value',0);
    set(handles.RBSaveGraphFig,'value',1);
    set(handles.RBSaveGraphPrem,'value',1);
    set(handles.RBSaveGraphTot,'value',0);
     
% --------------------------------------------------------------------
function varargout = SousMenuGraphLoad_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.PPNomVar.
    
%///////////////////////////////////////////////////////////////
%         Saisie des informations relatives au modele
%///////////////////////////////////////////////////////////////
% --------------------------------------------------------------------
function ENomVar_Callback(hObject, eventdata, handles)
%affiche les parametres de la variable en question
entree=handles.VarEntree;
[nvar,toto]=size(entree);
nomvar=get(handles.ENomVar,'string');
for i=1:nvar
    entree(i,1);
    nomvar(2:length(nomvar));
    if strcmp(nomvar(2:length(nomvar)),num2str(entree(i,1)))==1
        set(handles.PPLoi,'value',entree(i,2));
        set(handles.EP1,'string',num2str(entree(i,3)));
        set(handles.EP2,'string',num2str(entree(i,4)));        
    end
 end
% --------------------------------------------------------------------
function ENbVar_Callback(hObject, eventdata, handles)
    % Stub for Callback of the uicontrol handles.PPNbVar.
    NbVariable=get(handles.ENbVar,'string');
    handles.NbVar=eval(NbVariable);
    %intialisation des entrees a U[0,1]
    handles.VarEntree=[];
    %mettre partie param sur X1
    set(handles.ENomVar,'string','X1');
    ENomVar_Callback(hObject, eventdata, handles)
    for i=1:handles.NbVar
        if handles.Correlation==0        
           handles.VarEntree=[handles.VarEntree;i 1 0 1];
        else
           handles.VarEntree=[handles.VarEntree;i 2 0 1];
        end
    end
    guidata(SAinterface,handles);
    if str2num(NbVariable)>9
       set(handles.RBToutOrdre,'value',0);
    end
    if str2num(NbVariable)>8
       set(handles.RBFast,'value',0);
    end
    
% --------------------------------------------------------------------
function varargout = EFonction_Callback(h, eventdata, handles, varargin)
    % Stub for Callback of the uicontrol handles.EFonction.
    loc=get(handles.EFonction,'string');
    p=get(handles.ENbVar,'string');
    %si on a des Xi dans la fonction, on les remplace par x(:,i)
	for i=p:-1:1
        loc2=findstr(loc,['X' num2str(i)]);
        while ~isempty(loc2)
        j=1;
            %decale la chaine pour inserer x(:,i)
            if (i<10)
                for k=length(loc):-1:loc2(j)+2
                loc(k+4)=loc(k);
                end   
            elseif(i<100)
                for k=length(loc):-1:loc2(j)+3
                loc(k+4)=loc(k);
                end   
            else
                for k=length(loc):-1:loc2(j)+4
                loc(k+4)=loc(k);
                end   
            end 
            %insertion du x(;,i)
            loc(loc2(j))='x';loc(loc2(j)+1)='(';
            loc(loc2(j)+2)=':';loc(loc2(j)+3)=',';
            loc3=num2str(i);
            if (i<10)
                loc(loc2(j)+4)=loc3;
                loc(loc2(j)+5)=')';
            elseif(i<100)
                loc(loc2(j)+4)=loc3(1);
                loc(loc2(j)+5)=loc3(2);            
                loc(loc2(j)+6)=')';
            else
                loc(loc2(j)+4)=loc3(1);
                loc(loc2(j)+5)=loc3(2);            
                loc(loc2(j)+6)=loc3(3);            
                loc(loc2(j)+7)=')';
            end
        loc2=findstr(loc,['X' num2str(i)]);    
        end
	end
	handles.Fonction=loc;
	guidata(SAinterface,handles);

% --------------------------------------------------------------------
function varargout = PPLoi_Callback(h, eventdata, handles, varargin)
loi=get(handles.PPLoi,'value');
if handles.Correlation==1    % si correlation  les variables  
set(handles.PPLoi,'value',2);% sont obligatoirement gaussiennes
end
if loi==1, nbpar=2; end
if loi==2, nbpar=2; end
if loi==3, nbpar=1; end
if loi==4, nbpar=2; end
if loi==5, nbpar=2; end
if loi==6, nbpar=1; end
if loi==7, nbpar=2; end
if loi==8, nbpar=1; end
if loi==9, nbpar=2; end
if nbpar==2
   set(handles.TP2,'Enable','on');
   set(handles.EP2,'Enable','on');
else
    set(handles.TP2,'Enable','off');
    set(handles.EP2,'Enable','off');
end

% --------------------------------------------------------------------
function varargout = EP1_Callback(h, eventdata, handles, varargin)
% --------------------------------------------------------------------
function varargout = EP2_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = PBSauve_Callback(h, eventdata, handles, varargin)
%loi=1 si uniforme et 2 si normale, etc (U,N,E,W,G,S,B,X?LN)....
loi=get(handles.PPLoi,'value');
nomvar=get(handles.ENomVar,'string');
var=eval(nomvar(2:length(nomvar)));
entree=handles.VarEntree;
[nvar,toto]=size(entree);
pb='nok';
for i=1:nvar
       if entree(i,1)==var, pb=i; end
end
if not(pb=='nok')
    Btnnom=questdlg('This input distribution is allready speciefied. Do you want to overwrite it ?');
    if strcmp(Btnnom,'Yes')==1
        if pb==1 
            entree=handles.VarEntree(2:nvar,:);
        else
            if pb==nvar
                entree=handles.VarEntree(1:nvar-1,:);
            else
            entree=handles.VarEntree([1:pb-1 pb+1:nvar],:);
            end
        end
        handles.VarEntree=[];
        handles.VarEntree=entree;
    else
        return;
    end 
end
%parametre 1
loc=get(handles.EP1,'string');
p1=str2num(loc);
%parametre 2 
loc=get(handles.EP2,'string');
p2=str2num(loc);
%sauvegarde dans une matrice
handles.VarEntree=[handles.VarEntree ; var loi p1 p2];
guidata(SAinterface,handles);

% --- Executes on button press in PBFcnHelp.
function PBFcnHelp_Callback(hObject, eventdata, handles)
msgbox('Write litteral function with variables Xi, or give subroutine name like myfunction(x), what is describe in myfunction.m');


%----------------------------------------------------
%                   Entrees Correlees
%----------------------------------------------------
function PopupCorrel_Callback(hObject, eventdata, handles)
% Executes on selection change in PopupCorrel.
loc=get(handles.PopupCorrel,'value');
Matrice=[zeros(handles.NbVar)];
for i=1:handles.NbVar
   Matrice(i,i)=1;
end
if isunix 
   temp=['./SATemp/VarianceMatrix'];
else
   temp=['SATemp\VarianceMatrix'];
end
save(temp,'Matrice','-ascii');
if loc==1
  handles.Correlation=0;
  handles.VarEntree=[];
  for i=1:handles.NbVar
     handles.VarEntree=[handles.VarEntree;i 1 0 1]; 
  end
  guidata(SAinterface,handles);
  %methode de Sobol uniquement disponible
  set(handles.RBSrc,'enable','on');
  set(handles.RBSrrc,'enable','on');
  set(handles.RBFast,'enable','on');
  set(handles.RBMckay,'enable','on');
  set(handles.RBPrcc,'enable','on');
  set(handles.RBPcc,'enable','on');
  set(handles.RBGam,'enable','on');
end
if loc==2 %cas des variables correlees
  % variables obligatoirement gaussiennes
  handles.Correlation=1;
  set(handles.PPLoi,'value',2);
  handles.VarEntree=[];
  for i=1:handles.NbVar
     handles.VarEntree=[handles.VarEntree;i 2 0 1]; 
  end
  guidata(SAinterface,handles);
  %methode de Sobol uniquement disponible
  set(handles.RBSrc,'enable','off');
  set(handles.RBSrrc,'enable','off');
  set(handles.RBFast,'enable','off');
  set(handles.RBMckay,'enable','off');
  set(handles.RBPrcc,'enable','off');
  set(handles.RBPcc,'enable','off');
  set(handles.RBGam,'enable','off');
  set(handles.RBSrc,'value',0);
  set(handles.RBSrrc,'value',0);
  set(handles.RBFast,'value',0);
  set(handles.RBMckay,'value',0);
  set(handles.RBPrcc,'value',0);
  set(handles.RBGam,'value',0);
  set(handles.RBPcc,'value',0);
  %echanillonnage classique uniquement
  set(handles.RBCrudeMC,'value',1);
  set(handles.RBQMC,'value',0);
  %demande de saisie de la matrice de variance
  if isunix 
     msgbox('Write covariance matrix in file SATemp/VarianceMatrix');
  else
     msgbox('Write covariance matrix in file SATemp\VarianceMatrix');
  end
end

%///////////////////////////////////////////////////////////////
%           saisie des informations relatives au calcul
%///////////////////////////////////////////////////////////////

% --------------------------------------------------------------------
function varargout = ENbPoint_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.ENbPoint.
loc=get(handles.ENbPoint,'string');
handles.NbPoint=str2num(loc);
guidata(SAinterface,handles);

% --------------------------------------------------------------------
function varargout = ENbCalcul_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.ENbCalcul.
loc=get(handles.ENbCalcul,'string');
handles.NbCalcul=str2num(loc);
guidata(SAinterface,handles);

% --------------------------------------------------------------------
function RBOrdreTot_Callback(hObject, eventdata, handles)
    loc=get(handles.RBOrdreTot,'value');
    loc2=get(handles.RBPremOrdre,'value');
    if loc==0, set(handles.RBToutOrdre,'value',0);
    end
    
% --------------------------------------------------------------------
function RBToutOrdre_Callback(hObject, eventdata, handles)
    loc=get(handles.RBToutOrdre,'value');
    if loc==1
        set(handles.RBOrdreTot,'value',1);
        set(handles.RBPremOrdre,'value',1);
    end
    
% --------------------------------------------------------------------
function RBPremOrdre_Callback(hObject, eventdata, handles)
    loc=get(handles.RBPremOrdre,'value');
    loc2=get(handles.RBOrdreTot,'value');
    if loc==0, set(handles.RBToutOrdre,'value',0); 
    elseif loc2==1,
        set(handles.RBToutOrdre,'value',1); 
    end
    
% --------------------------------------------------------------------
function RBCrudeMC_Callback(hObject, eventdata, handles)
    if handles.Correlation==1
       set(handles.RBQMC,'value',0);
       set(handles.RBCrudeMC,'value',1);
    end
    loc=get(handles.RBCrudeMC,'value');
    if loc==1
        set(handles.RBQMC,'value',0);
    else
        set(handles.RBQMC,'value',1);
    end
    
% --------------------------------------------------------------------
function RBQMC_Callback(hObject, eventdata, handles)
    if handles.Correlation==1
       set(handles.RBQMC,'value',0);
       set(handles.RBCrudeMC,'value',1);
    end
    loc=get(handles.RBQMC,'value');
    if loc==1
        set(handles.RBCrudeMC,'value',0);
    else
        set(handles.RBCrudeMC,'value',1);
    end

% --------------------------------------------------------------------
function varargout = RBSobol_Callback(h, eventdata, handles, varargin)
    p=handles.NbVar;
    loc=get(handles.RBSobol,'value');
    loc1=get(handles.RBSrc,'value');
    loc2=get(handles.RBSrrc,'value');
    loc3=get(handles.RBFast,'value');
    loc4=get(handles.RBMckay,'value');
    loc5=get(handles.RBPrcc,'value');
    loc6=get(handles.RBGam,'value');
    loc7=get(handles.RBPcc,'value');
    if loc==1&loc1==0&loc2==0&loc4==0&loc5==0&loc6==0&loc7==0
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBOrdreTot,'enable','on');
        if p<10&loc3==0
            set(handles.RBToutOrdre,'enable','on');
        else
            set(handles.RBToutOrdre,'enable','off');
            set(handles.RBToutOrdre,'value',0);
        end
    end
    %apparition du choix du systme d'chantillonnage
    if loc==1
        ValeurVisible='on';
    else
        ValeurVisible='off';
    end
    set(handles.FSampling,'visible',ValeurVisible);
    set(handles.TSamplingVisual,'visible',ValeurVisible);
    set(handles.RBCrudeMC,'visible',ValeurVisible);
    set(handles.RBQMC,'visible',ValeurVisible);
   
% --------------------------------------------------------------------
function varargout = RBSrc_Callback(h, eventdata, handles, varargin)
    loc=get(handles.RBSrc,'value');
    if loc==1,
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBOrdreTot,'enable','off');
        set(handles.RBOrdreTot,'value',0);
        set(handles.RBToutOrdre,'enable','off');
        set(handles.RBToutOrdre,'value',0);
    else
        RBSobol_Callback(h, eventdata, handles, varargin);
        RBFast_Callback(h, eventdata, handles, varargin);
    end 
      
% --------------------------------------------------------------------
function varargout = RBSrrc_Callback(h, eventdata, handles, varargin)
    loc=get(handles.RBSrrc,'value');
    if loc==1,
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBOrdreTot,'enable','off');
        set(handles.RBOrdreTot,'value',0);
        set(handles.RBToutOrdre,'enable','off');
        set(handles.RBToutOrdre,'value',0);
    else
        RBSobol_Callback(h, eventdata, handles, varargin);
        RBFast_Callback(h, eventdata, handles, varargin);
    end 
    
% --------------------------------------------------------------------
function varargout = RBFast_Callback(h, eventdata, handles, varargin)
    loc=get(handles.RBFast,'value');
    loc2=get(handles.RBSrc,'value');
    loc3=get(handles.RBSrrc,'value');
    loc4=get(handles.RBMckay,'value');
    loc5=get(handles.RBPrcc,'value');
    loc6=get(handles.RBGam,'value');
    loc7=get(handles.RBPcc,'value');
    if loc==1&loc2==0&loc3==0&loc4==0&loc5==0&loc6==0&loc7==0
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBOrdreTot,'enable','on');
        set(handles.RBToutOrdre,'enable','off');
        set(handles.RBToutOrdre,'value',0);
        set(handles.ENbCalcul,'value',1);
        set(handles.ENbCalcul,'string',num2str(1));
    else
       RBSobol_Callback(h, eventdata, handles, varargin); 
    end
    p=handles.NbVar;
    if p>8
        set(handles.RBFast,'value',0);
        uiwait(warndlg('FAST method not available for more than  8 inputs'));
        return
    end
    
% --------------------------------------------------------------------
function varargout = RBMckay_Callback(h, eventdata, handles, varargin)
    loc=get(handles.RBMckay,'value');
    if loc==1,
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBOrdreTot,'enable','off');
        set(handles.RBOrdreTot,'value',0);
        set(handles.RBToutOrdre,'enable','off');
        set(handles.RBToutOrdre,'value',0);
    else
        RBSobol_Callback(h, eventdata, handles, varargin);
        RBFast_Callback(h, eventdata, handles, varargin);
    end 
        
% --------------------------------------------------------------------
function RBPrcc_Callback(h, eventdata, handles, varargin)
    loc=get(handles.RBPrcc,'value');
    if loc==1,
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBOrdreTot,'enable','off');
        set(handles.RBOrdreTot,'value',0);
        set(handles.RBToutOrdre,'enable','off');
        set(handles.RBToutOrdre,'value',0);
    else
        RBSobol_Callback(h, eventdata, handles, varargin);
        RBFast_Callback(h, eventdata, handles, varargin);
    end 
    
% --------------------------------------------------------------------
function RBGam_Callback(h, eventdata, handles, varargin)
    loc=get(handles.RBGam,'value');
    if loc==1,
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBToutOrdre,'enable','off');
        set(handles.RBOrdreTot,'value',0);
        set(handles.RBToutOrdre,'value',0);
        set(handles.RBOrdreTot,'enable','off');
    else
        RBSobol_Callback(h, eventdata, handles, varargin);
        RBFast_Callback(h, eventdata, handles, varargin);
    end 
    
% --------------------------------------------------------------------
function RBPcc_Callback(h, eventdata, handles, varargin)
    loc=get(handles.RBPcc,'value');
    if loc==1,
        set(handles.RBPremOrdre,'enable','on');
        set(handles.RBOrdreTot,'enable','off');
        set(handles.RBOrdreTot,'value',0);
        set(handles.RBToutOrdre,'enable','off');
        set(handles.RBToutOrdre,'value',0);
    else
        RBSobol_Callback(h, eventdata, handles, varargin);
        RBFast_Callback(h, eventdata, handles, varargin);
    end 


%///////////////////////////////////////////////////////////////
%                Calcul des indices de sensibilite
%///////////////////////////////////////////////////////////////

% --------------------------------------------------------------------
function varargout = PBCalcul_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.PBCalcul.
% ----
% saisie du nb d'entrees et de la fonction 
% au cas ou changements non pris en compte
ENbVar_Callback(h, eventdata, handles);
EFonction_Callback(h, eventdata, handles);
% verification que toutes les entrees sont renseignees
p=handles.NbVar;
n=handles.NbPoint;
NbCal=handles.NbCalcul;
handles.VarEntree=sortrows(handles.VarEntree);
guidata(SAinterface,handles);
VVarEntree=handles.VarEntree;
FFonction=handles.Fonction;
InfoEntree='ok';
%affichage des informations sur le modele
disp('////////////////////////////////////////////////////////');
loc=get(handles.EFonction,'string');
disp(loc);
SousMenuReca_Callback(h, eventdata, handles, varargin);
disp('////////////////////////////////////////////////////////');
%test de validite des entrees du modele
if not(size(handles.VarEntree)==[p 4])
    InfoEntree='nok'; 
end
if strcmp(InfoEntree,'nok')==1
    errordlg('you don''t give all input distributions');
    return;
end
%  saisie du nom de fichier de resultat
toto=get(handles.EFichierResultat,'String');
if isempty(toto)==1
    errordlg('you don''t give result file name');
    return;
end
% mise 0 du PM de visualisation graphique
ListeMethode='';
% ordre des indices
locPrem=get(handles.RBPremOrdre,'value');
locTot=get(handles.RBOrdreTot,'value');
locTout=get(handles.RBToutOrdre,'value');
% calcul par Sobol si demande
loc=get(handles.RBSobol,'value');
if loc==1 & handles.Correlation==0
    %-----schema d'echantillonnage
    locSampling=1; %CrudeMC
    loc=get(handles.RBQMC,'value');
    if loc==1
       locSampling=2;%QMC
    end
    %------        
    disp('Sobol method');
    ListeMethode=strvcat(ListeMethode,'Sobol');
    set(handles.PBCalcul,'backgroundcolor','r');
    [loc1,loc2,loc3]=Sobol(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction,locSampling);
    set(handles.PBCalcul,'backgroundcolor','g');
    %affichage des indices
    if locTout==1
        disp('All order indices (increasing order and total indices)');
        disp(loc3);
    else
        if locTot==1
        disp('Total order sensitivity indices');
        disp(loc2);
        end
        if locPrem==1
        disp('First order sensitivity indices');
        disp(loc1);
        end
    end
    handles.ResultatSobol=loc1;
    handles.ResultatSobolT=loc2;
    loc3=loc3(2:end,:);
    handles.ResultatSobolTout=loc3;
    guidata(SAinterface,handles);
    %sauvegarde dans un fichier
    if locPrem==1
        if isunix 
            tutu=['./Resultats/' toto '.sobol'];
        else
            tutu=['Resultats\' toto '.sobol'];
        end
        save(tutu,'loc1','-ascii');
    end
    if locTot==1
        if isunix 
            tutu=['./Resultats/' toto '.sobolT'];
        else
            tutu=['Resultats\' toto '.sobolT'];
        end
        save(tutu,'loc2','-ascii');
    end
    if locTout==1
        if isunix 
            tutu=['./Resultats/' toto '.sobolTout'];
        else
            tutu=['Resultats\' toto '.sobolTout'];
        end
        save(tutu,'loc3','-ascii');
    end
end
% calcul par Sobol avec entrees correelees
if loc==1 & handles.Correlation==1
    %--recuperation de la matrice de correlation
    if isunix 
        temp=['./SATemp/VarianceMatrix'];
    else
        temp=['SATemp\VarianceMatrix'];
    end
    Matrice=load(temp,'-ascii');
    %--schema d'echantillonnage
    locSampling=1; %CrudeMC
    loc=get(handles.RBQMC,'value');
    if loc==1
       locSampling=2;%QMC
    end
    %--        
    disp('Sobol method for correlated inputs');
    ListeMethode=strvcat(ListeMethode,'Sobol');
    set(handles.PBCalcul,'backgroundcolor','r');
    [loc1,loc2,loc3]=Sobol_Correl(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction,locSampling,Matrice);
    set(handles.PBCalcul,'backgroundcolor','g');    
    %----Definition des groupes de variables correlees---
    groupe=[];
    liste=[1:p];
    liste2=[];
    for i=liste
        if length(find(i==liste2))==0
            Indices=zeros(1,p);
            tmp=find(~Matrice(i,:)==0);
            Indices(1:length(tmp))=tmp;
            groupe=[groupe; Indices];
            liste2=[liste2 Indices];
        end    
    end
    %---------------------------------
    [p_ind,tmp]=size(groupe);
    handles.p_ind=p_ind;
    guidata(SAinterface,handles);
    %affichage des indices
    disp('Composition of groups by individual variables');
    for l=1:p_ind
              g=groupe(l,:);
              I_g=find(~g==0);
              disp(['group number ' num2str(l) ' contains variables : ' num2str(g(I_g))]);
    end
    if locTout==1
        disp('All order group sensitivity indices (increasing order and total indices)');
        disp(loc3);
    else
        if locTot==1
        disp('Total order group sensitivity indices');
        disp(loc2);
        end
        if locPrem==1
        disp('First order group sensitivity indices');
        disp(loc1);
        end
    end
    handles.ResultatSobol=loc1;
    handles.ResultatSobolT=loc2;
    loc3=loc3(2:end,:);
    handles.ResultatSobolTout=loc3;
    guidata(SAinterface,handles);
    %sauvegarde dans un fichier
    if locPrem==1
        if isunix 
            tutu=['./Resultats/' toto '.sobol'];
        else
            tutu=['Resultats\' toto '.sobol'];
        end
        fid = fopen(tutu,'w');
        fprintf(fid,'Composition of groups by individual variables\n');
        for l=1:p_ind
              g=groupe(l,:);
              I_g=find(~g==0);
              tmp=['group number ' num2str(l) ' contains variables : ' num2str(g(I_g))];
              fprintf(fid,tmp,'a');
              fprintf(fid,'\n','a');
        end
        tmp='First order group sensitivity indices';
        fprintf(fid,tmp,'a');
        fprintf(fid,'\n','a');
        fclose(fid);
        save(tutu,'loc1','-ascii','-append');
    end
    if locTot==1
        if isunix 
            tutu=['./Resultats/' toto '.sobolT'];
        else
            tutu=['Resultats\' toto '.sobolT'];
        end
        fid = fopen(tutu,'w');
        fprintf(fid,'Composition of groups by individual variables\n');
        for l=1:p_ind
              g=groupe(l,:);
              I_g=find(~g==0);
              tmp=['group number ' num2str(l) ' contains variables : ' num2str(g(I_g))];
              fprintf(fid,tmp,'a');
              fprintf(fid,'\n','a');
        end
        tmp='Total order group sensitivity indices';
        fprintf(fid,tmp,'a');
        fprintf(fid,'\n','a');
        fclose(fid);
        save(tutu,'loc2','-ascii','-append');
    end
    if locTout==1
        if isunix 
            tutu=['./Resultats/' toto '.sobolTout'];
        else
            tutu=['Resultats\' toto '.sobolTout'];
        end
        fid = fopen(tutu,'w');
        fprintf(fid,'Composition of groups by individual variables\n');
        for l=1:p_ind
              g=groupe(l,:);
              I_g=find(~g==0);
              tmp=['group number ' num2str(l) ' contains variables : ' num2str(g(I_g))];
              fprintf(fid,tmp,'a');
              fprintf(fid,'\n','a');
        end
        tmp='All order group sensitivity indices';
        fprintf(fid,tmp,'a');
        fprintf(fid,'\n','a');
        fclose(fid);
        save(tutu,'loc3','-ascii','-append');
    end
end
% calcul par Fast si demande
loc=get(handles.RBFast,'value');
if loc==1
    disp('FAST method')
    ListeMethode=strvcat(ListeMethode,'Fast');
    set(handles.PBCalcul,'backgroundcolor','r');
    [loc1,loc2]=Fast(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction);
    set(handles.PBCalcul,'backgroundcolor','g');
    handles.ResultatFast=loc1;
    handles.ResultatFastT=loc2;
    guidata(SAinterface,handles);
%sauvegarde dans un fichier
    if locPrem==1
        if isunix 
            tutu=['./Resultats/' toto '.fast'];
        else
            tutu=['Resultats\' toto '.fast'];
        end
        save(tutu,'loc1','-ascii');
    end
    if locTot==1
        if isunix 
            tutu=['./Resultats/' toto '.fastT'];
        else
            tutu=['Resultats\' toto '.fastT'];
        end
        save(tutu,'loc2','-ascii');
    end
end
%calcul par SRC si demande
loc=get(handles.RBSrc,'value');
if loc==1
    disp('SRC indices')
    ListeMethode=strvcat(ListeMethode,'SRC');
    set(handles.PBCalcul,'backgroundcolor','r'); 
    loc1=Src(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction);
    set(handles.PBCalcul,'backgroundcolor','g');
    handles.ResultatSrc=loc1;
    guidata(SAinterface,handles);
%sauvegarde dans un fichier
    if isunix 
            tutu=['./Resultats/' toto '.src'];
    else
            tutu=['Resultats\' toto '.src'];
    end
    save(tutu,'loc1','-ascii');
end
%calcul par SRRC si demande
loc=get(handles.RBSrrc,'value');
if loc==1
    disp('SRRC indices')
    ListeMethode=strvcat(ListeMethode,'SRRC');
    set(handles.PBCalcul,'backgroundcolor','r');
    loc1=Srrc(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction);
    set(handles.PBCalcul,'backgroundcolor','g');
    handles.ResultatSrrc=loc1;
    guidata(SAinterface,handles);
    %sauvegarde dans un fichier
    if isunix 
            tutu=['./Resultats/' toto '.srrc'];
    else
            tutu=['Resultats\' toto '.srrc'];
    end
    save(tutu,'loc1','-ascii');
end
loc=get(handles.RBMckay,'value');
if loc==1
     disp('McKay method')
     ListeMethode=strvcat(ListeMethode,'McKay');
     set(handles.PBCalcul,'backgroundcolor','r');
     loc1=Mckay(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction);
     set(handles.PBCalcul,'backgroundcolor','g');
     handles.ResultatMckay=loc1;
     guidata(SAinterface,handles);
     %sauvegarde dans un fichier
     if isunix 
            tutu=['./Resultats/' toto '.mckay'];
     else
            tutu=['Resultats\' toto '.mckay'];
     end
     save(tutu,'loc1','-ascii');
end
loc=get(handles.RBGam,'value');
if loc==1
     disp('GAM indices')
     warning('off'); 
     ListeMethode=strvcat(ListeMethode,'GAM');
     set(handles.PBCalcul,'backgroundcolor','r');
     loc1=Gam(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction);
     set(handles.PBCalcul,'backgroundcolor','g');
     handles.ResultatGam=loc1;
     guidata(SAinterface,handles);
     %sauvegarde dans un fichier
     if isunix 
            tutu=['./Resultats/' toto '.gam'];
     else
            tutu=['Resultats\' toto '.gam'];
     end
     save(tutu,'loc1','-ascii');
end
loc=get(handles.RBPcc,'value');
if loc==1
     disp('PCC indices')
     ListeMethode=strvcat(ListeMethode,'PCC');
     set(handles.PBCalcul,'backgroundcolor','r');
     loc1=Pcc(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction);
     set(handles.PBCalcul,'backgroundcolor','g');
     handles.ResultatPcc=loc1;
     guidata(SAinterface,handles);
     %sauvegarde dans un fichier
     if isunix 
            tutu=['./Resultats/' toto '.pcc'];
     else
            tutu=['Resultats\' toto '.pcc'];
     end
     save(tutu,'loc1','-ascii');
end
loc=get(handles.RBPrcc,'value');
if loc==1
     disp('PRCC indices')
     ListeMethode=strvcat(ListeMethode,'PRCC');
     set(handles.PBCalcul,'backgroundcolor','r');
     loc1=Prcc(NbCal,locPrem,locTot,locTout,p,n,VVarEntree,FFonction);
     set(handles.PBCalcul,'backgroundcolor','g');
     handles.ResultatPrcc=loc1;
     guidata(SAinterface,handles);
     %sauvegarde dans un fichier
     if isunix 
            tutu=['./Resultats/' toto '.prcc'];
     else
            tutu=['Resultats\' toto '.prcc'];
     end
     save(tutu,'loc1','-ascii');
end
set(handles.PPVisualGraphMethode,'string',ListeMethode);
if strcmp(ListeMethode,'')==0, handles.ExistenceCalcul=1; end
guidata(SAinterface,handles);


%/////////////////////////////////////////////////////////////
%                         Graphiques
%/////////////////////////////////////////////////////////////

%effacement des graph lors de l'activation de PPVisualGraphMethode.
function PPVisualGraphMethode_Callback(hObject, eventdata, handles)
    axes(handles.GraphTot);
    cla;
    set(handles.GraphTot,'Visible','off');
    axes(handles.GraphPrem);
    cla;
    set(handles.GraphPrem,'Visible','off');

function PBGraph_Callback(hObject, eventdata, handles)
    %enleve ancien graph
    loc=get(handles.GraphTot,'Visible');
    loc2=get(handles.GraphPrem,'Visible');
    if strcmp(loc,'off')==0|strcmp(loc2,'off')==0 
        Btngraph=questdlg('Delete hold graph ?');
        if strcmp(Btngraph,'Yes')==1
            axes(handles.GraphTot);
            cla;
            set(handles.GraphTot,'Visible','off');
            axes(handles.GraphPrem);
            cla;
            set(handles.GraphPrem,'Visible','off');   
        end
    end
    var=get(handles.PPVisualGraphMethode,'value');
    %LocOrdre=1 pour indice d'ordre 1 et 2 pour indices totaux
    if handles.Correlation
        p_g=handles.p_ind;
    else
        p_g = handles.NbVar;
    end
    LocIndex = get(handles.PPVisualGraphMethode,'Value');
    LocListeMethode = get(handles.PPVisualGraphMethode,'String');
    LocMethode = deblank(LocListeMethode(LocIndex,:));
    if strcmp(LocMethode,'Sobol')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatSobol;
        title('1st order indices, Sobol');
        hold on;
        for i=1:p_g
            errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
        end
        hold off;
        LocResultat=handles.ResultatSobolT;
        if ~isempty(LocResultat)
            axes(handles.GraphTot);
            set(handles.GraphTot,'Visible','on');
            axis([0 p_g+1 0 1]);
            %indices totaux
            title('total indices, Sobol');
            hold on;
            for i=1:p_g
                errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
            end
            hold off;
        end
    end  
    if strcmp(LocMethode,'Fast')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatFast;
        title('1st order indices, Fast');
        hold on;
        for i=1:p_g
            plot(i,LocResultat(i,2),'r.');
        end
        hold off;
        %indices totaux
        LocResultat=handles.ResultatFastT;
        if ~isempty(LocResultat)
            axes(handles.GraphTot);
            set(handles.GraphTot,'Visible','on');
            axis([0 p_g+1 0 1]);
            title('total indices, FastExtended');
            hold on;
            for i=1:p_g
                plot(i,LocResultat(i,2),'r.');
            end
            hold off;
        end
    end
    if strcmp(LocMethode,'SRC')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatSrc;
        title('SRC indices');
        hold on;
        for i=1:p_g
            errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
        end
        hold off;
  end
  if strcmp(LocMethode,'SRRC')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatSrrc;
        title('SRRC indices ');
        hold on;
        for i=1:p_g
            errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
        end
        hold off;
   end
   if strcmp(LocMethode,'McKay')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatMckay;
        title('McKay indices');
        hold on;
        for i=1:p_g
            errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
        end
        hold off;
        end
   if strcmp(LocMethode,'PCC')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatPcc;
        title('PCC indices');
        hold on;
        for i=1:p_g
            errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
        end
        hold off;
   end
   if strcmp(LocMethode,'PRCC')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatPrcc;
        title('PRCC indices');
        hold on;
        for i=1:p_g
            errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
        end
        hold off;
        end
        if strcmp(LocMethode,'GAM')==1
        axes(handles.GraphPrem);
        set(handles.GraphPrem,'Visible','on');
        axis([0 p_g+1 0 1]);
        %indices 1er ordre
        LocResultat=handles.ResultatGam;
        title('GAM based indices');
        hold on;
        for i=1:p_g
            errorbar(i,LocResultat(i,2),LocResultat(i,3).*2,'r.');
        end
        hold off;
   end
   
% --- Executes on button press in PBSaveGraph.
function PBSaveGraph_Callback(hObject, eventdata, handles)
    nomfich=get(handles.ESaveGraph,'string');
    if isunix
        nomfich=['./Resultats/' nomfich];
    else
        nomfich=['Resultats\' nomfich];
    end
    epsT=get(handles.RBSaveGraphEps,'value');
    figT=get(handles.RBSaveGraphFig,'value');
    jpgT=get(handles.RBSaveGraphJpg,'value');
    nomgraph='GraphPrem';
    totT=get(handles.RBSaveGraphTot,'value');
    if totT==1
        nomgraph='GraphTot';
    end
    %cree une figure intermediaire de sauvegarde
    %et y copie le graph voulu
    h=findobj(SAinterface,'Type','axes','tag',nomgraph);
    f=figure('visible','on');
    a=axes;
    pos=get(a,'position');
    un=get(a,'units');
    delete(a);
    tmp=copyobj(h,f);
    set(tmp,'Units',un);
    set(tmp,'Position',pos);
    %enregistre la figure intermediaire de sauvegarde
    if epsT==1, saveas(f,nomfich,'eps'); end
    if figT==1, saveas(f,nomfich,'fig'); end
    if jpgT==1, saveas(f,nomfich,'jpg'); end
    %efface la figure intermediaire de sauvegarde
    delete(f);
    % invisibilite de la fiche de sauvegarde
    set(handles.FSaveGraph,'Visible','off');
    set(handles.TSaveGraph,'Visible','off');
    set(handles.RBSaveGraphPrem,'Visible','off');
    set(handles.RBSaveGraphTot,'Visible','off');
    set(handles.RBSaveGraphEps,'Visible','off');
    set(handles.RBSaveGraphJpg,'Visible','off');
    set(handles.RBSaveGraphFig,'Visible','off');
    set(handles.TSaveGraph2,'Visible','off');
    set(handles.ESaveGraph,'Visible','off');
    set(handles.PBSaveGraph,'Visible','off');
    
% --------------------------------------------------------------------
function RBSaveGraphPrem_Callback(hObject, eventdata, handles)
    set(handles.RBSaveGraphTot,'value',0);
% --------------------------------------------------------------------
function RBSaveGraphTot_Callback(hObject, eventdata, handles)
    set(handles.RBSaveGraphPrem,'value',0);
% --------------------------------------------------------------------
function RBSaveGraphFig_Callback(hObject, eventdata, handles)
    set(handles.RBSaveGraphEps,'value',0);
    set(handles.RBSaveGraphJpg,'value',0);
% --------------------------------------------------------------------
function RBSaveGraphEps_Callback(hObject, eventdata, handles)
    set(handles.RBSaveGraphFig,'value',0);
    set(handles.RBSaveGraphJpg,'value',0);
% --------------------------------------------------------------------
function RBSaveGraphJpg_Callback(hObject, eventdata, handles)
    set(handles.RBSaveGraphEps,'value',0);
    set(handles.RBSaveGraphFig,'value',0);
    
% /////////////////////////////////////////////////////////////////////
%       creation du repertoire Resultats/ si il n'existe pas
% /////////////////////////////////////////////////////////////////////
function EFichierResultat_Callback(hObject, eventdata, handles)
    warning off MATLAB:MKDIR:DirectoryExists
    loc=mkdir('Resultats');
    
% /////////////////////////////////////////////////////////////////////
%                 fonctions qui ne font rien
% /////////////////////////////////////////////////////////////////////
% --------------------------------------------------------------------
function varargout = MenuOutils_Callback(h, eventdata, handles, varargin)
% --------------------------------------------------------------------
function varargout = EFichierResult_Callback(h, eventdata, handles, varargin)
% --------------------------------------------------------------------
function EFichierResultat_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function PPOrdreInd_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function ENbPoint_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function EP2_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function EP1_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function PPLoi_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function PPNomVar_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function EFonction_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function PPNbVar_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function ENbCalcul_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function PPVisualGraphMethode_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function ENomVar_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function ENbVar_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function Untitled_1_Callback(hObject, eventdata, handles)
% --------------------------------------------------------------------
function ESaveGraph_CreateFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function ESaveGraph_Callback(hObject, eventdata, handles)
% --------------------------------------------------------------------
function CBSobol_Callback(hObject, eventdata, handles)
% --------------------------------------------------------------------
function ENomVar_ButtonDownFcn(hObject, eventdata, handles)
% --------------------------------------------------------------------
function PopupCorrel_CreateFcn(hObject, eventdata, handles)
% Executes during object creation, after setting all properties.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end
